package org.bitbucket.bradleysmithllc.etlunit.io.file;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.github.fge.jackson.JsonLoader;
import com.github.fge.jsonschema.exceptions.ProcessingException;
import com.github.fge.jsonschema.main.JsonSchema;
import com.github.fge.jsonschema.main.JsonSchemaFactory;
import com.github.fge.jsonschema.report.ProcessingReport;
import org.bitbucket.bradleysmithllc.etlunit.feature.ResourceFeatureMetaInfo;
import org.bitbucket.bradleysmithllc.etlunit.util.*;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.regex.Pattern;

class FlatFileType implements DataFormatValidator {
	private final String id;
	private final Pattern pattern;

	private static final JsonSchema fileTypesSchema = JsonSchemaLoader.fromResource("org/bitbucket/bradleysmithllc/etlunit/io/file/ffml/ffmlTypes.jsonSchema");

	public FlatFileType(String id, JsonNode node) {
		this.id = id;
		this.pattern = Pattern.compile(node.asText());
	}

	public String getId() {
		return id;
	}

	public Pattern getPattern() {
		return pattern;
	}

	public static Map<String, FlatFileType> load(String types) {
		Map<String, FlatFileType> typeMap = new HashMap<String, FlatFileType>();

		try {
			JsonNode jsonNode = JsonLoader.fromString(types);

			ProcessingReport pr = fileTypesSchema.validate(jsonNode);

			if (!pr.isSuccess())
			{
				throw new IllegalArgumentException(pr.toString());
			}

			ObjectNode on = (ObjectNode) jsonNode;

			Iterator<Map.Entry<String, JsonNode>> fields = on.fields();

			while (fields.hasNext()) {
				Map.Entry<String, JsonNode> field = fields.next();

				typeMap.put(field.getKey(), new FlatFileType(field.getKey(), field.getValue()));
			}

			return typeMap;
		} catch (Exception e1) {
			throw new IllegalArgumentException(e1);
		}
	}
}