package org.bitbucket.bradleysmithllc.etlunit.parser;

import com.fasterxml.jackson.databind.JsonNode;

import java.util.List;
import java.util.Map;

public interface ETLTestValueObject extends ETLTestDebugTraceable
{
	boolean isNull();

	enum value_type
	{
		quoted_string,
		object,
		list,
		literal,
		pojo,
		t_null
	}

	String getValueAsString();

	boolean getValueAsBoolean();

	List<ETLTestValueObject> getValueAsList();

	List<String> getValueAsListOfStrings();

	Map<String, ETLTestValueObject> getValueAsMap();

	Object getValueAsPojo();

	value_type getValueType();

	ETLTestValueObject query(String path);

	ETLTestValueObject queryRequired(String path);

	enum merge_type
	{
		/**
		 * If there is a conflict, the left value (this) wins
		 */
		left_merge,

		/**
		 * If there is a conflict, the right value (other) wins
		 */
		right_merge,
	}

	enum merge_policy
	{
		recursive,
		nonrecursive
	}

	/**
	 * Perform a recursive left merge.
	 * @param obj
	 * @return
	 */
	ETLTestValueObject merge(ETLTestValueObject obj);

	ETLTestValueObject merge(ETLTestValueObject object, merge_type type, merge_policy policy);

	ETLTestValueObject copy();

	JsonNode getJsonNode();
}